"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (_ref) {
    var t = _ref.types;

    return {
        visitor: {
            Identifier: function Identifier(path, _ref2) {
                var options = _ref2.opts;

                if (!isCorrectIdentifier(path)) return;

                var identifier = path.node,
                    scope = path.scope;


                if (isDefined(identifier, scope)) return;

                var declarations = options.declarations;


                if (!isArray(declarations)) return;

                declarations.some(handleDeclaration, { path: path, identifier: identifier });
            }
        }
    };

    function isCorrectIdentifier(path) {
        var parentPath = path.parentPath;


        if (parentPath.isDeclaration()) return false;

        if (parentPath.findParent(isImportDeclaration)) return false;

        if (parentPath.isMemberExpression() && parentPath.get("object") != path) return false;

        return true;
    }

    function isDefined(identifier, _ref3) {
        var bindings = _ref3.bindings,
            parent = _ref3.parent;

        var variables = Object.keys(bindings);

        if (variables.some(has, identifier)) return true;

        return parent ? isDefined(identifier, parent) : false;
    }

    function has(identifier) {
        var name = this.name;


        return identifier == name;
    }

    function handleDeclaration(declaration) {
        var path = this.path,
            identifier = this.identifier;


        if (!declaration) return;

        var importType = null;

        if (hasDefault(declaration, identifier)) {
            importType = ImportType.DEFAULT;
        } else if (hasMember(declaration, identifier)) {
            importType = ImportType.MEMBER;
        }

        if (importType) {
            var program = path.findParent(isProgram);

            insertImport(program, identifier, importType, declaration.path);

            return true;
        }
    }

    function hasDefault(declaration, identifier) {
        return declaration["default"] == identifier.name;
    }

    function hasMember(declaration, identifier) {
        var members = isArray(declaration.members) ? declaration.members : [];

        return members.some(has, identifier);
    }

    function insertImport(program, identifier, type, moduleSource) {
        var programBody = program.get("body");

        var currentImportDeclarations = programBody.reduce(toImportDeclarations, []);

        var importDidAppend = currentImportDeclarations.some(addToImportDeclaration, { identifier: identifier, type: type, moduleSource: moduleSource });

        if (!importDidAppend) {
            var specifier = void 0;

            if (type == ImportType.DEFAULT) {
                specifier = t.importDefaultSpecifier(identifier);
            } else if (type == ImportType.MEMBER) {
                specifier = t.importSpecifier(identifier, identifier);
            }

            var importDeclaration = t.importDeclaration([specifier], t.stringLiteral(moduleSource));

            program.unshiftContainer("body", importDeclaration);
        }
    }

    function isProgram(path) {
        return path.isProgram();
    }

    function isImportDeclaration(path) {
        return path.isImportDeclaration();
    }

    function toImportDeclarations(list, currentPath) {
        if (currentPath.isImportDeclaration()) list.push(currentPath);

        return list;
    }

    function addToImportDeclaration(importDeclarationPath) {
        var identifier = this.identifier,
            type = this.type,
            moduleSource = this.moduleSource;
        var node = importDeclarationPath.node;


        if (node.source.value != moduleSource) return false;

        var specifiers = node.specifiers;


        if (type == ImportType.DEFAULT) {
            if (!specifiers.some(hasImportDefaultSpecifier)) {
                var specifier = t.importDefaultSpecifier(identifier);

                importDeclarationPath.unshiftContainer("specifiers", specifier);

                return true;
            }
        }

        if (type == ImportType.MEMBER) {
            if (!specifiers.some(hasSpecifierWithName, identifier)) {
                var _specifier = t.importSpecifier(identifier, identifier);

                importDeclarationPath.pushContainer("specifiers", _specifier);

                return true;
            }
        }
    }

    function hasImportDefaultSpecifier(node) {
        return t.isImportDefaultSpecifier(node);
    }

    function hasSpecifierWithName(node) {
        if (!t.isImportSpecifier(node)) return false;

        var name = this.name;


        return node.imported.name == name;
    }
};

var isArray = require("is-array");

var ImportType = {
    DEFAULT: 1,
    MEMBER: 2
};